#include "Ellipse.h"
#include <math.h>


#define PI 3.141592654

Elipse::Elipse(EditPanel *editPanel,
                 Coord trans,
                 double a,
                 double b,
                 double alfa,
                 wxString name,
                 wxColor color):
    GeomObj(editPanel, trans, name, color)
{
    this->trans = trans;
    this->a = a;
    this->b = b;
    this->alfa = alfa;
}

Elipse::Elipse(EditPanel *editPanel,
                 Coord fokus1,
                 Coord fokus2 ,
                 Coord point,
                 wxString name,
                 wxColor color):
    GeomObj(editPanel, point, name, color)
{
    Coord S((fokus1.x + fokus2.x)/2, (fokus1.y + fokus2.y)/2);
    b = sqrt((S.x - point.x)*(S.x - point.x) + (S.y - point.y)*(S.y - point.y));
    a = sqrt((fokus1.x - point.x)*(fokus1.x - point.x) + (fokus1.y - point.y)*(fokus1.y - point.y));

    trans = S;

    double s = (fokus1.y - fokus2.y);

    if (s >= 0)
         alfa = acos((fokus1.x - fokus2.x)
                /(sqrt((fokus1.x - fokus2.x)*(fokus1.x - fokus2.x)
                 + (fokus1.y - fokus2.y)*(fokus1.y - fokus2.y))));

    if (s < 0)
        alfa = 2*PI - acos((fokus1.x - fokus2.x)
                /(sqrt((fokus1.x - fokus2.x)*(fokus1.x - fokus2.x)
                 + (fokus1.y - fokus2.y)*(fokus1.y - fokus2.y))));
}

void Elipse::Paint(wxDC& dc)
{
    if (!visibility)
        return;
    int tmpx = pos.ToScreenX(editPanel);
    int tmpy = pos.ToScreenY(editPanel);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(wxPen(color,width));

    double t = 0;
    int num =  10*((a > b)? a : b)/editPanel->zoom;
    double x,y;
    dc.DrawPoint(wxPoint(trans.ToScreenX(editPanel), trans.ToScreenY(editPanel)));
    for (int i = 0; i < num; ++i)
    {
        x = a*cos(t)*cos(alfa) - b*sin(t)*sin(alfa) + trans.x;
        y = a*cos(t)*sin(alfa) + b*sin(t)*cos(alfa) + trans.y;
        Coord tmp(x, y);
        dc.DrawPoint(wxPoint(tmp.ToScreenX(editPanel), tmp.ToScreenY(editPanel)));
        t = t + (2*PI)/num;
    }

//    dc.DrawElipse(wxCoord(tmpx), wxCoord(tmpy), a / editPanel->zoom, b / editPanel->zoom);
    if (label)
    {
        dc.SetTextForeground(color);
        Coord pos2 = Coord(x , y);
        dc.DrawText(name, pos2.ToScreenX(editPanel), pos2.ToScreenY(editPanel));
    }
}

void Elipse::Paint(wxDC& dc, wxColor newColor)
{
    if (!visibility)
        return;
    int tmpx = pos.ToScreenX(editPanel);
    int tmpy = pos.ToScreenY(editPanel);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(wxPen(newColor,width));

    double t = 0;
    int num =  10*((a > b)? a : b)/editPanel->zoom;
    double x,y;
    dc.DrawPoint(wxPoint(trans.ToScreenX(editPanel), trans.ToScreenY(editPanel)));
    for (int i = 0; i < num; ++i)
    {
        x = a*cos(t)*cos(alfa) - b*sin(t)*sin(alfa) + trans.x;
        y = a*cos(t)*sin(alfa) + b*sin(t)*cos(alfa) + trans.y;
        Coord tmp(x, y);
        dc.DrawPoint(wxPoint(tmp.ToScreenX(editPanel), tmp.ToScreenY(editPanel)));
        t = t + (2*PI)/num;
    }
//    dc.DrawElipse(wxCoord(tmpx), wxCoord(tmpy), a / editPanel->zoom, b / editPanel->zoom);
    if (label)
    {
        dc.SetTextForeground(color);
        Coord pos2 = Coord(x, y);
        dc.DrawText(name, pos2.ToScreenX(editPanel), pos2.ToScreenY(editPanel));
    }
}


objectType Elipse::GetType()
{
    return oELLIPSE;
}

wxString Elipse::GetName()
{
    return name;
}

wxString Elipse::GetText()
{
    wxString text = name + wxS("; ") + name + wxS(" je elipsa");
    return text;
}

Coord Elipse::GetPos()
{
    return trans;
}

double Elipse::GetA()
{
    return a;
}

double Elipse::GetB()
{
    return b;
}

double Elipse::GetAlfa()
{
    return alfa;
}

