#include "Hyperbole.h"
#include <math.h>


#define PI 3.141592654


Hyperbole::Hyperbole(EditPanel *editPanel,
                     Coord trans,
                     double a,
                     double b,
                     double alfa,
                     wxString name,
                     wxColor color):
    GeomObj(editPanel, trans, name, color)
{
    this->trans = trans;
    this->a = a;
    this->b = b;
    this->alfa = alfa;
}

Hyperbole::Hyperbole(EditPanel *editPanel,
                     Coord fokus1,
                     Coord fokus2 ,
                     Coord point,
                     wxString name,
                     wxColor color):
    GeomObj(editPanel, point, name, color)
{
    Coord S((fokus1.x + fokus2.x)/2, (fokus1.y + fokus2.y)/2);
    b = sqrt((S.x - point.x)*(S.x - point.x) + (S.y - point.y)*(S.y - point.y));
    double e = sqrt((fokus1.x - S.x)*(fokus1.x - S.x) + (fokus1.y - S.y)*(fokus1.y - S.y));

    a = sqrt(-b*b + e*e);

    trans = S;
     double s = (fokus1.y - fokus2.y);

    if (s >= 0)
         alfa = acos((fokus1.x - fokus2.x)
                /(sqrt((fokus1.x - fokus2.x)*(fokus1.x - fokus2.x)
                 + (fokus1.y - fokus2.y)*(fokus1.y - fokus2.y))));

    if (s < 0)
        alfa = 2*PI - acos((fokus1.x - fokus2.x)
                /(sqrt((fokus1.x - fokus2.x)*(fokus1.x - fokus2.x)
                 + (fokus1.y - fokus2.y)*(fokus1.y - fokus2.y))));

}

void Hyperbole::Paint(wxDC& dc)
{
    if (!visibility)
        return;
    int tmpx = pos.ToScreenX(editPanel);
    int tmpy = pos.ToScreenY(editPanel);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(wxPen(color,width));

//    int tmpW, tmpH;
//    dc.GetSize(&tmpW, &tmpH);
//
    double t = -PI;
    //int num =  10*((a > b)? a : b)/editPanel->zoom;
    int num = 10000;
    double x;
    double y;

    dc.DrawPoint(wxPoint(trans.ToScreenX(editPanel), trans.ToScreenY(editPanel)));
    for (int i = 0; i < num; ++i)
    {
        x = a*coshl(t)*cos(alfa) - b*sinhl(t)*sin(alfa) + trans.x;
        y = a*coshl(t)*sin(alfa) + b*sinhl(t)*cos(alfa) + trans.y;

        double xx = -a*coshl(t)*cos(alfa) - b*sinhl(t)*sin(alfa) + trans.x;
        double yy = -a*coshl(t)*sin(alfa) + b*sinhl(t)*cos(alfa) + trans.y;

        Coord tmp(x, y);
        dc.DrawPoint(wxPoint(tmp.ToScreenX(editPanel), tmp.ToScreenY(editPanel)));
        Coord tmp2(xx, yy);
        dc.DrawPoint(wxPoint(tmp2.ToScreenX(editPanel), tmp2.ToScreenY(editPanel)));

        t = t + (2*PI)/num;
    }

//    dc.DrawElipse(wxCoord(tmpx), wxCoord(tmpy), a / editPanel->zoom, b / editPanel->zoom);
    if (label)
    {
        dc.SetTextForeground(color);
        Coord pos2 = Coord(x, y);
        dc.DrawText(name, pos2.ToScreenX(editPanel), pos2.ToScreenY(editPanel));
    }
}

void Hyperbole::Paint(wxDC& dc, wxColor newColor)
{
    if (!visibility)
        return;
    int tmpx = pos.ToScreenX(editPanel);
    int tmpy = pos.ToScreenY(editPanel);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(wxPen(newColor,width));

    double t = -PI;
    int num = 10000;
    double x;
    double y;

    dc.DrawPoint(wxPoint(trans.ToScreenX(editPanel), trans.ToScreenY(editPanel)));
    for (int i = 0; i < num; ++i)
    {
        x = a*coshl(t)*cos(alfa) - b*sinhl(t)*sin(alfa) + trans.x;
        y = a*coshl(t)*sin(alfa) + b*sinhl(t)*cos(alfa) + trans.y;

        double xx = -a*coshl(t)*cos(alfa) - b*sinhl(t)*sin(alfa) + trans.x;
        double yy = -a*coshl(t)*sin(alfa) + b*sinhl(t)*cos(alfa) + trans.y;

        Coord tmp(x, y);
        dc.DrawPoint(wxPoint(tmp.ToScreenX(editPanel), tmp.ToScreenY(editPanel)));
        Coord tmp2(xx, yy);
        dc.DrawPoint(wxPoint(tmp2.ToScreenX(editPanel), tmp2.ToScreenY(editPanel)));

        t = t + (2*PI)/num;
    }
    if (label)
    {
        dc.SetTextForeground(color);
        Coord pos2 = Coord(x, y);
        dc.DrawText(name, pos2.ToScreenX(editPanel), pos2.ToScreenY(editPanel));
    }
}


objectType Hyperbole::GetType()
{
    return oHYPERBOLE;
}

wxString Hyperbole::GetName()
{
    return name;
}

wxString Hyperbole::GetText()
{
    wxString text = name + wxS("; ") + name + wxS(" je hyperbola");
    return text;
}

Coord Hyperbole::GetPos()
{
    return trans;
}

double Hyperbole::GetAlfa()
{
    return alfa;
}

double Hyperbole::GetA()
{
    return a;
}

double Hyperbole::GetB()
{
    return b;
}

