#include "Parabola.h"
#include <math.h>


#define PI 3.141592654

Parabola::Parabola(EditPanel *editPanel,
                  Coord trans,
                  double p,
                  double alfa,
                  wxString name,
                  wxColor color):
    GeomObj(editPanel, trans, name, color)
{
    this->trans = trans;
    this->p = p;
    this->alfa = alfa;
}
Parabola::Parabola(EditPanel *editPanel,
                   Coord fokus,
                   Coord point,
                   wxString name,
                   wxColor color):
    GeomObj(editPanel, fokus, name, color)
{
    this->p = 2*(sqrt((fokus.x - point.x)*(fokus.x - point.x) + (fokus.y - point.y)*(fokus.y - point.y)));

    double s = (fokus.y - point.y);

    if (s >= 0)
        alfa = acos((fokus.x - point.x) /(p/2));
    if (s < 0)
        alfa = 2*PI-acos((fokus.x - point.x) /(p/2));

    trans = point;
}

void Parabola::Paint(wxDC& dc)
{
    if (!visibility)
        return;
    int tmpx = pos.ToScreenX(editPanel);
    int tmpy = pos.ToScreenY(editPanel);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(wxPen(color,width));

    int num = 60000;
    double t = -(num/2000);
    double x,y;
    dc.DrawPoint(wxPoint(trans.ToScreenX(editPanel), trans.ToScreenY(editPanel)));
    for (int i = 0; i < num; ++i)
    {
        x =  (p/2)*t*t*cos(alfa) - p*t*sin(alfa) + trans.x;
        y =  (p/2)*t*t*sin(alfa) + p*t*cos(alfa) + trans.y;

        Coord tmp(x, y);
        dc.DrawPoint(wxPoint(tmp.ToScreenX(editPanel), tmp.ToScreenY(editPanel)));
        t = t + 1.0/1000;
    }
//    Coord tmp(trans.x - (p/2), -100);
//    Coord tmp2(trans.x - (p/2), 100);
//    dc.DrawLine( wxCoord(tmp.ToScreenX(editPanel)),  wxCoord(tmp.ToScreenY(editPanel)),
//                wxCoord(tmp2.ToScreenX(editPanel)), wxCoord(tmp2.ToScreenY(editPanel)));

//    dc.DrawElipse(wxCoord(tmpx), wxCoord(tmpy), a / editPanel->zoom, b / editPanel->zoom);
    if (label)
    {
        dc.SetTextForeground(color);
        Coord pos2 = Coord(x, y);
        dc.DrawText(name, pos2.ToScreenX(editPanel), pos2.ToScreenY(editPanel));
    }
}

void Parabola::Paint(wxDC& dc, wxColor newColor)
{
    if (!visibility)
        return;
    int tmpx = pos.ToScreenX(editPanel);
    int tmpy = pos.ToScreenY(editPanel);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(wxPen(newColor,width));

    int num = 60000;
    double t = -(num/2000);
    double x,y;
    dc.DrawPoint(wxPoint(trans.ToScreenX(editPanel), trans.ToScreenY(editPanel)));
    for (int i = 0; i < num; ++i)
    {
        x =  (p/2)*t*t*cos(alfa) - p*t*sin(alfa) + trans.x;
        y =  (p/2)*t*t*sin(alfa) + p*t*cos(alfa) + trans.y;

        Coord tmp(x, y);
        dc.DrawPoint(wxPoint(tmp.ToScreenX(editPanel), tmp.ToScreenY(editPanel)));
        t = t + 1.0/1000;
    }
    if (label)
    {
        dc.SetTextForeground(color);
        Coord pos2 = Coord(x, y);
        dc.DrawText(name, pos2.ToScreenX(editPanel), pos2.ToScreenY(editPanel));
    }
}


objectType Parabola::GetType()
{
    return oPARABOLA;
}

wxString Parabola::GetName()
{
    return name;
}

wxString Parabola::GetText()
{
    wxString text = name + wxS("; ") + name + wxS(" je parabola");
    return text;
}

double Parabola::GetP()
{
    return p;
}

double Parabola::GetAlfa()
{
    return alfa;
}

Coord Parabola::GetPos()
{
    return trans;
}

